// Copyright (C) 2002 H.Okada(suigyodo) All rights reserved.

#include <qlistbox.h>
#include <qlineedit.h>
#include <qfileinfo.h>
#include <qstring.h>
#include <qmessagebox.h>
#include <qlayout.h>

#include <string.h>
#include <assert.h>
#include <list>
#include <string>
using namespace std;
#include "xbschglobal.h"
#include "selcompimpl.h"



#include "coord.h"
#include "complib.h"

/* 
 *  Constructs a selcomp which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
SelComp::SelComp( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : selcompBase( parent, name, modal, fl )
{
	m_pCompIndex = NULL;
	m_nLib = -1;
	m_nIndex = -1;
	ComponentView = new SComponentView(this,false);
	ComponentView->setProperty( "minimumSize", QSize( 200, 0 ) );
	selcompBaseLayout->addMultiCellWidget(ComponentView,1,2,2,2);
}

/*  
 *  Destroys the object and frees any allocated resources
 */
SelComp::~SelComp()
{
    // no need to delete child widgets, Qt does it all for us
}

void SelComp::libraryChanged()
{
	int nLib =ListBoxLibrary->currentItem();
	setComponentName(nLib);
}

void SelComp::componentChanged()
{
	QString name = ListBoxName->currentText();
	LineEditName->setText(name);
	string strName = name.latin1();
	int l,i;
	const SCompIndex* pIndex = g_SearchComponentIndex(strName.c_str(),&l,&i,NULL);
	ComponentView->setComponentIndex(pIndex);
}

int SelComp::exec()
{
	setLibraryName();
	setComponentName(0);
	return selcompBase::exec();
}


void SelComp::setLibraryName()
{
	ListBoxLibrary->clear();	
	int nCount = g_LibraryCount();
	if(nCount==0) return;
	int n;
	for(n=0;n<nCount;n++){
		const SCompLib* pLib = g_LibraryInfo(n);
		const char* name = pLib->libraryName();//̾ϥѥ̾
		QFileInfo finfo(QString::fromLocal8Bit(name));
		QString baseName = finfo.baseName(); 
		ListBoxLibrary->insertItem(baseName);
	}
	ListBoxLibrary->setSelected (0,true);
}

void SelComp::setComponentName(int nLib)
{
	ListBoxName->clear();
	const SCompLib* pLib = g_LibraryInfo(nLib);
	if(pLib == NULL) return;
	//Ƥʤο
	int nCount = pLib->count();
	int n;
	for(n=0;n<nCount;n++){
		SCompIndex* pCIndex = pLib->compIndex(n);
		const char* name = pCIndex->name();
		ListBoxName->insertItem(QString::fromLocal8Bit(name));
	}
	ListBoxName->setSelected (0,true);
	ListBoxName->ensureCurrentVisible();
}

void SelComp::accept()
{
	QString qsName = LineEditName->text();
	string strName = string(qsName.local8Bit());
	int nLib;
	int nIndex;
	const SCompIndex* pCompIndex = g_SearchComponentIndex(strName.c_str(),&nLib,&nIndex,NULL);
	if(pCompIndex != NULL){
		m_strComponentName = strName;
		m_nLib = nLib;
		m_nIndex = nIndex;
		m_pCompIndex = pCompIndex;
		selcompBase::accept();
	}else{
		QString msg ="\"";
		msg += qsName;
		msg += "\" is not found. Continue?";
		int n = QMessageBox::warning (	this,
										"Qt-BSch",
										msg,
										"OK",
										"Cancel",
										QString::null,
										0,
										1 );
		if(n==1){
			selcompBase::reject();
		}
	}
}



