// Copyright (C) 2002 H.Okada(suigyodo) All rights reserved.
#include <qlistbox.h>
#include <qpushbutton.h>
#include <qfiledialog.h>
#include <qstringlist.h>

#include "xbsch.h"
#include "setlibimpl.h"


/* 
 *  Constructs a SetLib which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
SetLib::SetLib( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : SetLibBase( parent, name, modal, fl )
{
}

/*  
 *  Destroys the object and frees any allocated resources
 */
SetLib::~SetLib()
{
    // no need to delete child widgets, Qt does it all for us
}


void SetLib::setFiles(const QStringList& files)
{
	m_files=QStringList(files);
}

QStringList SetLib::selectedFiles()
{
	return m_files;
}


int SetLib::exec()
{
	ListBox1->clear();
	for(QStringList::Iterator it = m_files.begin(); it != m_files.end(); ++it ){
		if(ListBox1->count() >= MAX_LIB){
			break;
		}
		ListBox1->insertItem(*it);
	}
	if(ListBox1->count() > 0){
		ListBox1->setSelected (0,true);
	}
	setButtonEnable();
	return SetLibBase::exec();
}

void SetLib::accept()
{
	m_files.clear();
	int count=ListBox1->count();
	if(count>MAX_LIB) count=MAX_LIB;
	for(int n=0; n<count; n++){
		m_files.append(ListBox1->text(n));
	}
	SetLibBase::accept();
}

void SetLib::OnAddButton()
{
	if(ListBox1->count() < MAX_LIB){
		QStringList sl=QFileDialog::getOpenFileNames("*.lb3");
		if(!sl.isEmpty()){
			for ( QStringList::Iterator it = sl.begin(); it != sl.end(); ++it ){
				if(ListBox1->count() >= MAX_LIB){
					break;
				}
				ListBox1->insertItem(*it);
			}
		}
	}
	//ҤȤĤǤ饤֥̾åȤƤ...
	if(ListBox1->count() > 0){
		ListBox1->setSelected (0,true);
	}
	setButtonEnable();

}

void SetLib::OnDownButton()
{
	int count = ListBox1->count();
	int index = ListBox1->currentItem();

	if(count>0 && index<(count-1)){
		QString str = ListBox1->text(index);
		ListBox1->removeItem(index);
		ListBox1->insertItem(str,index+1);
		ListBox1->setSelected (index+1,true);
	}
	setButtonEnable();
}

void SetLib::OnUpButton()
{
	int count = ListBox1->count();
	int index = ListBox1->currentItem();

	if(count>0 && index>=1){
		QString str = ListBox1->text(index);
		ListBox1->removeItem(index);
		ListBox1->insertItem(str,index-1);
		ListBox1->setSelected (index-1,true);
	}
	setButtonEnable();
}

void SetLib::itemChanged()
{
	setButtonEnable();
}

void SetLib::OnRemoveButton()
{
	if(ListBox1->count() == 0){
		buttonRemove->setEnabled(false);
		return;
	}
	int index = ListBox1->currentItem(); 
	if(index != -1){
		ListBox1->removeItem(index);
	}
	setButtonEnable();
}

void SetLib::setButtonEnable()
{
	int count = ListBox1->count();
	int index = ListBox1->currentItem();

	buttonAdd->setEnabled(count<MAX_LIB);
	buttonRemove->setEnabled(count>0 && index!=-1);
	buttonUp->setEnabled(count>0 && index>=1);
	buttonDown->setEnabled(count>0 && index<(count-1));
}
